@@ -60,7 +60,8 @@ typedef enum {
 	NAUTILUS_FILE_SORT_BY_TYPE = 2,
 	NAUTILUS_FILE_SORT_BY_MTIME = 3,
 	NAUTILUS_FILE_SORT_BY_ATIME = 4,
-	NAUTILUS_FILE_SORT_BY_STARRED = 5,
+	NAUTILUS_FILE_SORT_BY_BTIME = 5,
+	NAUTILUS_FILE_SORT_BY_STARRED = 6,
 
 	/* The following are specific to special locations and as such are not to be
 	 * included in the "org.gnome.nautilus.SortOrder" enum.
@@ -196,6 +197,7 @@ gboolean                nautilus_file_can_get_size                      (Nautilu
 goffset                 nautilus_file_get_size                          (NautilusFile                   *file);
 time_t                  nautilus_file_get_mtime                         (NautilusFile                   *file);
 time_t                  nautilus_file_get_atime                         (NautilusFile                   *file);
+time_t                  nautilus_file_get_btime                         (NautilusFile                   *file);
 time_t                  nautilus_file_get_recency                       (NautilusFile                   *file);
 time_t                  nautilus_file_get_trash_time                    (NautilusFile                   *file);
 GFileType               nautilus_file_get_file_type                     (NautilusFile                   *file);
@@ -419,9 +421,7 @@ char *                  nautilus_file_get_string_attribute_with_default_q (Nauti
 gboolean                nautilus_file_matches_uri                       (NautilusFile                   *file,
 									 const char                     *uri);
 
-/* Is the file local? */
-gboolean                nautilus_file_is_local                          (NautilusFile                   *file);
-gboolean                nautilus_file_is_local_or_fuse                  (NautilusFile                   *file);
+gboolean                nautilus_file_has_local_path                    (NautilusFile                   *file);
 
 /* Comparing two file objects for sorting */
 NautilusFileSortType    nautilus_file_get_default_sort_type             (NautilusFile                   *file,
@@ -529,6 +529,7 @@ struct NautilusFile {
 typedef enum {
 	NAUTILUS_DATE_TYPE_MODIFIED,
 	NAUTILUS_DATE_TYPE_ACCESSED,
+	NAUTILUS_DATE_TYPE_CREATED,
 	NAUTILUS_DATE_TYPE_TRASHED,
 	NAUTILUS_DATE_TYPE_RECENCY
 } NautilusDateType;
