@@ -91,6 +91,8 @@ struct _NautilusToolbar
     GtkWidget *view_toggle_button;
     GtkWidget *view_toggle_icon;
 
+    GtkWidget *app_menu;
+
     GtkWidget *operations_popover;
     GtkWidget *operations_container;
     GtkWidget *operations_revealer;
@@ -128,7 +130,7 @@ enum
 
 static GParamSpec *properties[NUM_PROPERTIES] = { NULL, };
 
-G_DEFINE_TYPE (NautilusToolbar, nautilus_toolbar, GTK_TYPE_HEADER_BAR);
+G_DEFINE_TYPE (NautilusToolbar, nautilus_toolbar, HDY_TYPE_HEADER_BAR);
 
 static void nautilus_toolbar_set_window_slot_real (NautilusToolbar    *self,
                                                    NautilusWindowSlot *slot);
@@ -159,9 +161,9 @@ toolbar_update_appearance (NautilusToolbar *self)
 }
 
 static void
-activate_back_or_forward_menu_item (GtkMenuItem    *menu_item,
-                                    NautilusWindow *window,
-                                    gboolean        back)
+activate_back_or_forward_menu_item (GtkMenuItem        *menu_item,
+                                    NautilusWindowSlot *window_slot,
+                                    gboolean            back)
 {
     int index;
 
@@ -169,21 +171,21 @@ activate_back_or_forward_menu_item (GtkMenuItem    *menu_item,
 
     index = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (menu_item), "user_data"));
 
-    nautilus_window_back_or_forward (window, back, index, nautilus_event_get_window_open_flags ());
+    nautilus_window_slot_back_or_forward (window_slot, back, index);
 }
 
 static void
 activate_back_menu_item_callback (GtkMenuItem     *menu_item,
                                   NautilusToolbar *self)
 {
-    activate_back_or_forward_menu_item (menu_item, self->window, TRUE);
+    activate_back_or_forward_menu_item (menu_item, self->window_slot, TRUE);
 }
 
 static void
 activate_forward_menu_item_callback (GtkMenuItem     *menu_item,
                                      NautilusToolbar *self)
 {
-    activate_back_or_forward_menu_item (menu_item, self->window, FALSE);
+    activate_back_or_forward_menu_item (menu_item, self->window_slot, FALSE);
 }
 
 static void
@@ -892,7 +894,7 @@ on_location_entry_focus_changed (GObject    *object,
 static void
 nautilus_toolbar_constructed (GObject *object)
 {
-    GtkBuilder *builder;
+    g_autoptr (GtkBuilder) builder = NULL;
     NautilusToolbar *self = NAUTILUS_TOOLBAR (object);
 
     builder = gtk_builder_new_from_resource ("/org/gnome/nautilus/ui/nautilus-toolbar-switcher.ui");
@@ -1230,6 +1232,7 @@ nautilus_toolbar_class_init (NautilusToolbarClass *klass)
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, view_button);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, view_toggle_button);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, view_toggle_icon);
+    gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, app_menu);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, back_button);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, forward_button);
     gtk_widget_class_bind_template_child (widget_class, NautilusToolbar, toolbar_switcher_container);
@@ -1297,8 +1300,11 @@ slot_on_extensions_background_menu_changed (NautilusToolbar    *self,
                                             GParamSpec         *param,
                                             NautilusWindowSlot *slot)
 {
+    g_autoptr (GMenuModel) menu = NULL;
+
+    menu = nautilus_window_slot_get_extensions_background_menu (slot);
     nautilus_path_bar_set_extensions_background_menu (NAUTILUS_PATH_BAR (self->path_bar),
-                                                      nautilus_window_slot_get_extensions_background_menu (slot));
+                                                      menu);
 }
 
 static void
@@ -1306,8 +1312,11 @@ slot_on_templates_menu_changed (NautilusToolbar    *self,
                                 GParamSpec         *param,
                                 NautilusWindowSlot *slot)
 {
+    g_autoptr (GMenuModel) menu = NULL;
+
+    menu = nautilus_window_slot_get_templates_menu (slot);
     nautilus_path_bar_set_templates_menu (NAUTILUS_PATH_BAR (self->path_bar),
-                                          nautilus_window_slot_get_templates_menu (slot));
+                                          menu);
 }
 
 static void
@@ -1496,13 +1505,9 @@ nautilus_toolbar_set_window_slot (NautilusToolbar    *self,
 gboolean
 nautilus_toolbar_is_menu_visible (NautilusToolbar *self)
 {
-    GtkPopover *popover;
-
     g_return_val_if_fail (NAUTILUS_IS_TOOLBAR (self), FALSE);
 
-    popover = GTK_POPOVER (gtk_menu_button_get_popover (GTK_MENU_BUTTON (self->view_button)));
-
-    return gtk_widget_is_visible (GTK_WIDGET (popover));
+    return gtk_widget_is_visible (self->app_menu);
 }
 
 gboolean
