@@ -61,7 +61,6 @@
 #include "nautilus-metadata.h"
 #include "nautilus-mime-actions.h"
 #include "nautilus-notebook.h"
-#include "nautilus-other-locations-window-slot.h"
 #include "nautilus-pathbar.h"
 #include "nautilus-profile.h"
 #include "nautilus-properties-window.h"
@@ -81,9 +80,9 @@ static void mouse_forward_button_changed (gpointer callback_data);
 static void use_extra_mouse_buttons_changed (gpointer callback_data);
 static void nautilus_window_initialize_actions (NautilusWindow *window);
 static GtkWidget *nautilus_window_ensure_location_entry (NautilusWindow *window);
-static void close_slot (NautilusWindow     *window,
-                        NautilusWindowSlot *slot,
-                        gboolean            remove_from_notebook);
+static void nautilus_window_back_or_forward (NautilusWindow *window,
+                                             gboolean        back,
+                                             guint           distance);
 
 /* Sanity check: highest mouse button value I could find was 14. 5 is our
  * lower threshold (well-documented to be the one of the button events for the
@@ -98,7 +97,7 @@ static void close_slot (NautilusWindow     *window,
 
 struct _NautilusWindow
 {
-    GtkApplicationWindow parent_instance;
+    HdyApplicationWindow parent_instance;
 
     GtkWidget *notebook;
 
@@ -106,7 +105,7 @@ struct _NautilusWindow
      * Both of them may never be NULL.
      */
     GList *slots;
-    NautilusWindowSlot *active_slot;
+    NautilusWindowSlot *active_slot; /* weak reference */
 
     GtkWidget *content_paned;
 
@@ -158,13 +157,12 @@ enum
 {
     SLOT_ADDED,
     SLOT_REMOVED,
-    ACTIVE_SELECTION_CHANGED,
     LAST_SIGNAL
 };
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
-G_DEFINE_TYPE (NautilusWindow, nautilus_window, GTK_TYPE_APPLICATION_WINDOW);
+G_DEFINE_TYPE (NautilusWindow, nautilus_window, HDY_TYPE_APPLICATION_WINDOW);
 
 static const struct
 {
@@ -222,7 +220,7 @@ action_go_home (GSimpleAction *action,
     window = NAUTILUS_WINDOW (user_data);
     home = g_file_new_for_path (g_get_home_dir ());
 
-    nautilus_window_open_location_full (window, home, nautilus_event_get_window_open_flags (), NULL, NULL);
+    nautilus_window_open_location_full (window, home, 0, NULL, NULL);
 
     g_object_unref (home);
 }
@@ -270,7 +268,7 @@ action_up (GSimpleAction *action,
         {
             nautilus_window_open_location_full (NAUTILUS_WINDOW (user_data),
                                                 parent,
-                                                nautilus_event_get_window_open_flags (),
+                                                0,
                                                 NULL, NULL);
         }
 
@@ -283,8 +281,7 @@ action_back (GSimpleAction *action,
              GVariant      *state,
              gpointer       user_data)
 {
-    nautilus_window_back_or_forward (NAUTILUS_WINDOW (user_data),
-                                     TRUE, 0, nautilus_event_get_window_open_flags ());
+    nautilus_window_back_or_forward (NAUTILUS_WINDOW (user_data), TRUE, 0);
 }
 
 static void
@@ -292,8 +289,7 @@ action_forward (GSimpleAction *action,
                 GVariant      *state,
                 gpointer       user_data)
 {
-    nautilus_window_back_or_forward (NAUTILUS_WINDOW (user_data),
-                                     FALSE, 0, nautilus_event_get_window_open_flags ());
+    nautilus_window_back_or_forward (NAUTILUS_WINDOW (user_data), FALSE, 0);
 }
 
 static void
@@ -466,18 +462,6 @@ on_slot_location_changed (NautilusWindowSlot *slot,
     }
 }
 
-
-static void
-on_slot_selection_changed (NautilusWindowSlot *slot,
-                           GParamSpec         *pspec,
-                           NautilusWindow     *window)
-{
-    if (nautilus_window_get_active_slot (window) == slot)
-    {
-        g_signal_emit (window, signals[ACTIVE_SELECTION_CHANGED], 0);
-    }
-}
-
 static void
 notebook_switch_page_cb (GtkNotebook    *notebook,
                          GtkWidget      *page,
@@ -504,8 +488,6 @@ connect_slot (NautilusWindow     *window,
 {
     g_signal_connect (slot, "notify::location",
                       G_CALLBACK (on_slot_location_changed), window);
-    g_signal_connect (slot, "notify::selection",
-                      G_CALLBACK (on_slot_selection_changed), window);
 }
 
 static void
@@ -515,65 +497,18 @@ disconnect_slot (NautilusWindow     *window,
     g_signal_handlers_disconnect_by_data (slot, window);
 }
 
-static NautilusWindowSlot *
-nautilus_window_create_slot (NautilusWindow *window,
-                             GFile          *location)
-{
-    NautilusFile *file = NULL;
-    NautilusWindowSlot *slot;
-
-    if (location)
-    {
-        file = nautilus_file_get (location);
-    }
-    /* If not file, assume we open the home directory. We will switch eventually
-     * to a different location if not.
-     */
-    if (file && nautilus_file_is_other_locations (file))
-    {
-        slot = NAUTILUS_WINDOW_SLOT (nautilus_other_locations_window_slot_new (window));
-    }
-    else
-    {
-        slot = nautilus_window_slot_new (window);
-    }
-
-    nautilus_file_unref (file);
-
-    return slot;
-}
-
 static NautilusWindowSlot *
 nautilus_window_create_and_init_slot (NautilusWindow          *window,
-                                      GFile                   *location,
                                       NautilusWindowOpenFlags  flags)
 {
     NautilusWindowSlot *slot;
 
-    slot = nautilus_window_create_slot (window, location);
+    slot = nautilus_window_slot_new (window);
     nautilus_window_initialize_slot (window, slot, flags);
 
     return slot;
 }
 
-static NautilusWindowSlot *
-replace_active_slot (NautilusWindow          *window,
-                     GFile                   *location,
-                     NautilusWindowOpenFlags  flags)
-{
-    NautilusWindowSlot *new_slot;
-    NautilusWindowSlot *active_slot;
-
-    new_slot = nautilus_window_create_and_init_slot (window, location, flags);
-    active_slot = nautilus_window_get_active_slot (window);
-    if (active_slot)
-    {
-        close_slot (window, active_slot, TRUE);
-    }
-
-    return new_slot;
-}
-
 void
 nautilus_window_initialize_slot (NautilusWindow          *window,
                                  NautilusWindowSlot      *slot,
@@ -610,13 +545,6 @@ nautilus_window_open_location_full (NautilusWindow          *window,
 {
     NautilusWindowSlot *active_slot;
     gboolean new_tab_at_end;
-    NautilusNavigationState *navigation_state = NULL;
-
-    /* The location owner can be one of the slots requesting to handle an
-     * unhandled location. But this slot can be destroyed when switching to
-     * a new slot. So keep the location alive.
-     */
-    g_object_ref (location);
 
     /* Assert that we are not managing new windows */
     g_assert (!(flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW));
@@ -638,13 +566,7 @@ nautilus_window_open_location_full (NautilusWindow          *window,
 
     if (target_slot == NULL || (flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB) != 0)
     {
-        target_slot = nautilus_window_create_and_init_slot (window, location, flags);
-    }
-    else if (!nautilus_window_slot_handles_location (target_slot, location))
-    {
-        navigation_state = nautilus_window_slot_get_navigation_state (active_slot);
-
-        target_slot = replace_active_slot (window, location, flags);
+        target_slot = nautilus_window_create_and_init_slot (window, flags);
     }
 
     /* Make the opened location the one active if we weren't ask for the
@@ -655,21 +577,7 @@ nautilus_window_open_location_full (NautilusWindow          *window,
         nautilus_window_set_active_slot (window, target_slot);
     }
 
-    if (navigation_state != NULL)
-    {
-        nautilus_window_slot_open_location_set_navigation_state (target_slot,
-                                                                 location, flags, selection,
-                                                                 NAUTILUS_LOCATION_CHANGE_STANDARD,
-                                                                 navigation_state, 0);
-
-        free_navigation_state (navigation_state);
-    }
-    else
-    {
-        nautilus_window_slot_open_location_full (target_slot, location, flags, selection);
-    }
-
-    g_object_unref (location);
+    nautilus_window_slot_open_location_full (target_slot, location, flags, selection);
 }
 
 static void
@@ -1244,7 +1152,7 @@ action_restore_tab (GSimpleAction *action,
 
     location = nautilus_file_get_location (data->file);
 
-    slot = nautilus_window_create_and_init_slot (window, location, flags);
+    slot = nautilus_window_create_and_init_slot (window, flags);
 
     nautilus_window_slot_open_location_full (slot, location, flags, NULL);
     nautilus_window_slot_restore_navigation_state (slot, data);
@@ -1328,7 +1236,7 @@ places_sidebar_populate_popup_cb (GtkPlacesSidebar *sidebar,
             menu_item = gtk_model_button_new ();
             gtk_actionable_set_action_name (GTK_ACTIONABLE (menu_item),
                                             "win.empty-trash");
-            g_object_set (menu_item, "text", _("Empty _Trash"), NULL);
+            g_object_set (menu_item, "text", _("Empty _Trash…"), NULL);
             gtk_container_add (GTK_CONTAINER (menu), menu_item);
             gtk_widget_show (menu_item);
 
@@ -1427,43 +1335,18 @@ nautilus_window_show_sidebar (NautilusWindow *window)
     setup_side_pane_width (window);
 }
 
-static inline NautilusWindowSlot *
-get_first_inactive_slot (NautilusWindow *window)
-{
-    GList *l;
-    NautilusWindowSlot *slot;
-
-    for (l = window->slots; l != NULL; l = l->next)
-    {
-        slot = NAUTILUS_WINDOW_SLOT (l->data);
-        if (slot != window->active_slot)
-        {
-            return slot;
-        }
-    }
-
-    return NULL;
-}
-
 void
 nautilus_window_slot_close (NautilusWindow     *window,
                             NautilusWindowSlot *slot)
 {
-    NautilusWindowSlot *next_slot;
     NautilusNavigationState *data;
 
     DEBUG ("Requesting to remove slot %p from window %p", slot, window);
-    if (window == NULL)
+    if (window == NULL || slot == NULL)
     {
         return;
     }
 
-    if (window->active_slot == slot)
-    {
-        next_slot = get_first_inactive_slot (window);
-        nautilus_window_set_active_slot (window, next_slot);
-    }
-
     data = nautilus_window_slot_get_navigation_state (slot);
     if (data != NULL)
     {
@@ -1490,7 +1373,7 @@ nautilus_window_sync_bookmarks (NautilusWindow *window)
     GFile *location;
 
     slot = window->active_slot;
-    location = nautilus_window_slot_get_location (slot);
+    location = slot != NULL ? nautilus_window_slot_get_location (slot) : NULL;
 
     if (location != NULL)
     {
@@ -1512,6 +1395,9 @@ nautilus_window_sync_location_widgets (NautilusWindow *window)
     gboolean enabled;
 
     slot = window->active_slot;
+    /* This function is called by the active slot. */
+    g_assert (slot != NULL);
+
     location = nautilus_window_slot_get_location (slot);
 
     /* Change the location bar and path bar to match the current location. */
@@ -1791,6 +1677,11 @@ nautilus_window_show_operation_notification (NautilusWindow *window,
     NautilusFile *folder;
     GFile *current_location;
 
+    if (window->active_slot == NULL)
+    {
+        return;
+    }
+
     current_location = nautilus_window_slot_get_location (window->active_slot);
     if (gtk_window_has_toplevel_focus (GTK_WINDOW (window)))
     {
@@ -2000,8 +1891,6 @@ setup_toolbar (NautilusWindow *window)
                              G_CALLBACK (location_entry_location_changed_callback), window, 0);
     g_signal_connect_object (location_entry, "cancel",
                              G_CALLBACK (location_entry_cancel_callback), window, 0);
-
-    gtk_window_set_titlebar (GTK_WINDOW (window), window->toolbar);
 }
 
 static void
@@ -2011,7 +1900,7 @@ notebook_page_removed_cb (GtkNotebook *notebook,
                           gpointer     user_data)
 {
     NautilusWindow *window = user_data;
-    NautilusWindowSlot *slot = NAUTILUS_WINDOW_SLOT (page), *next_slot;
+    NautilusWindowSlot *slot = NAUTILUS_WINDOW_SLOT (page);
     gboolean dnd_slot;
 
     dnd_slot = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (slot), "dnd-window-slot"));
@@ -2020,12 +1909,6 @@ notebook_page_removed_cb (GtkNotebook *notebook,
         return;
     }
 
-    if (window->active_slot == slot)
-    {
-        next_slot = get_first_inactive_slot (window);
-        nautilus_window_set_active_slot (window, next_slot);
-    }
-
     close_slot (window, slot, FALSE);
 }
 
@@ -2260,7 +2143,7 @@ nautilus_window_constructed (GObject *self)
      * some actions trigger UI widgets to show/hide. */
     nautilus_window_initialize_actions (window);
 
-    slot = nautilus_window_create_and_init_slot (window, NULL, 0);
+    slot = nautilus_window_create_and_init_slot (window, 0);
     nautilus_window_set_active_slot (window, slot);
 
     window->bookmarks_id =
@@ -2324,7 +2207,7 @@ nautilus_window_destroy (GtkWidget *object)
     /* the slots list should now be empty */
     g_assert (window->slots == NULL);
 
-    window->active_slot = NULL;
+    g_clear_weak_pointer (&window->active_slot);
 
     g_clear_signal_handler (&window->bookmarks_id, nautilus_application_get_bookmarks (application));
 
@@ -2392,33 +2275,51 @@ nautilus_window_finalize (GObject *object)
 static void
 nautilus_window_save_geometry (NautilusWindow *window)
 {
+    GdkWindow *gdk_window;
+    GdkWindowState window_state;
     gboolean is_maximized;
 
     g_assert (NAUTILUS_IS_WINDOW (window));
 
-    if (gtk_widget_get_window (GTK_WIDGET (window)))
+    gdk_window = gtk_widget_get_window (GTK_WIDGET (window));
+
+    if (!gdk_window)
+    {
+        return;
+    }
+
+    window_state = gdk_window_get_state (gtk_widget_get_window (GTK_WIDGET (window)));
+
+    /* Don't save the window state for tiled windows. This is a special case,
+     * where the geometry only makes sense in combination with other tiled
+     * windows, that we can't possibly restore. */
+    if (window_state & GDK_WINDOW_STATE_TILED)
+    {
+        return;
+    }
+
+    is_maximized = window_state & GDK_WINDOW_STATE_MAXIMIZED;
+
+    /* Only save the initial size when the window is not maximized. If the
+     * window is maximized, a previously stored initial size will be more
+     * appropriate when unmaximizing the window in the future. */
+    if (!is_maximized)
     {
         gint width;
         gint height;
         GVariant *initial_size;
 
         gtk_window_get_size (GTK_WINDOW (window), &width, &height);
-
         initial_size = g_variant_new_parsed ("(%i, %i)", width, height);
-        is_maximized = gdk_window_get_state (gtk_widget_get_window (GTK_WIDGET (window)))
-                       & GDK_WINDOW_STATE_MAXIMIZED;
 
-        if (!is_maximized)
-        {
-            g_settings_set_value (nautilus_window_state,
-                                  NAUTILUS_WINDOW_STATE_INITIAL_SIZE,
-                                  initial_size);
-        }
-
-        g_settings_set_boolean
-            (nautilus_window_state, NAUTILUS_WINDOW_STATE_MAXIMIZED,
-            is_maximized);
+        g_settings_set_value (nautilus_window_state,
+                              NAUTILUS_WINDOW_STATE_INITIAL_SIZE,
+                              initial_size);
     }
+
+    g_settings_set_boolean (nautilus_window_state,
+                            NAUTILUS_WINDOW_STATE_MAXIMIZED,
+                            is_maximized);
 }
 
 void
@@ -2462,7 +2363,7 @@ nautilus_window_set_active_slot (NautilusWindow     *window,
         nautilus_toolbar_set_window_slot (NAUTILUS_TOOLBAR (window->toolbar), NULL);
     }
 
-    window->active_slot = new_slot;
+    g_set_weak_pointer (&window->active_slot, new_slot);
 
     /* make new slot active, if it exists */
     if (new_slot)
@@ -2472,7 +2373,6 @@ nautilus_window_set_active_slot (NautilusWindow     *window,
         nautilus_toolbar_set_window_slot (NAUTILUS_TOOLBAR (window->toolbar), new_slot);
 
         on_location_changed (window);
-        g_signal_emit (window, signals[ACTIVE_SELECTION_CHANGED], 0);
     }
 }
 
@@ -2535,7 +2435,8 @@ nautilus_window_key_press_event (GtkWidget   *widget,
         return GDK_EVENT_STOP;
     }
 
-    if (nautilus_window_slot_handle_event (window->active_slot, (GdkEvent *) event))
+    if (window->active_slot != NULL &&
+        nautilus_window_slot_handle_event (window->active_slot, (GdkEvent *) event))
     {
         return GDK_EVENT_STOP;
     }
@@ -2703,6 +2604,21 @@ nautilus_window_delete_event (GtkWidget   *widget,
     return FALSE;
 }
 
+static void
+nautilus_window_back_or_forward (NautilusWindow *window,
+                                 gboolean        back,
+                                 guint           distance)
+{
+    NautilusWindowSlot *slot;
+
+    slot = nautilus_window_get_active_slot (window);
+
+    if (slot != NULL)
+    {
+        nautilus_window_slot_back_or_forward (slot, back, distance);
+    }
+}
+
 static void
 on_multi_press_gesture_pressed (GtkGestureMultiPress *gesture,
                                 gint                  n_press,
@@ -2720,11 +2636,11 @@ on_multi_press_gesture_pressed (GtkGestureMultiPress *gesture,
 
     if (mouse_extra_buttons && (button == mouse_back_button))
     {
-        nautilus_window_back_or_forward (window, TRUE, 0, 0);
+        nautilus_window_back_or_forward (window, TRUE, 0);
     }
     else if (mouse_extra_buttons && (button == mouse_forward_button))
     {
-        nautilus_window_back_or_forward (window, FALSE, 0, 0);
+        nautilus_window_back_or_forward (window, FALSE, 0);
     }
 }
 
@@ -2789,8 +2705,6 @@ nautilus_window_init (NautilusWindow *window)
     gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (window)),
                                  "nautilus-window");
 
-    window->toolbar = nautilus_toolbar_new ();
-
     window_group = gtk_window_group_new ();
     gtk_window_group_add_window (window_group, GTK_WINDOW (window));
     g_object_unref (window_group);
@@ -2839,6 +2753,7 @@ nautilus_window_class_init (NautilusWindowClass *class)
 
     gtk_widget_class_set_template_from_resource (wclass,
                                                  "/org/gnome/nautilus/ui/nautilus-window.ui");
+    gtk_widget_class_bind_template_child (wclass, NautilusWindow, toolbar);
     gtk_widget_class_bind_template_child (wclass, NautilusWindow, content_paned);
     gtk_widget_class_bind_template_child (wclass, NautilusWindow, sidebar);
     gtk_widget_class_bind_template_child (wclass, NautilusWindow, places_sidebar);
@@ -2872,13 +2787,6 @@ nautilus_window_class_init (NautilusWindowClass *class)
                       NULL, NULL,
                       g_cclosure_marshal_VOID__OBJECT,
                       G_TYPE_NONE, 1, NAUTILUS_TYPE_WINDOW_SLOT);
-    signals[ACTIVE_SELECTION_CHANGED] =
-        g_signal_new ("active-selection-changed",
-                      G_TYPE_FROM_CLASS (class),
-                      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
-                      0,
-                      NULL, NULL, NULL,
-                      G_TYPE_NONE, 0);
 
     g_signal_connect_swapped (nautilus_preferences,
                               "changed::" NAUTILUS_PREFERENCES_MOUSE_BACK_BUTTON,
@@ -2908,30 +2816,6 @@ nautilus_window_new (GdkScreen *screen)
                          NULL);
 }
 
-NautilusWindowOpenFlags
-nautilus_event_get_window_open_flags (void)
-{
-    NautilusWindowOpenFlags flags = 0;
-    GdkEvent *event;
-
-    event = gtk_get_current_event ();
-
-    if (event == NULL)
-    {
-        return flags;
-    }
-
-    if ((event->type == GDK_BUTTON_PRESS || event->type == GDK_BUTTON_RELEASE) &&
-        (event->button.button == GDK_BUTTON_MIDDLE))
-    {
-        flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB;
-    }
-
-    gdk_event_free (event);
-
-    return flags;
-}
-
 void
 nautilus_window_show_about_dialog (NautilusWindow *window)
 {
@@ -2942,44 +2826,7 @@ nautilus_window_show_about_dialog (NautilusWindow *window)
     };
     const gchar *authors[] =
     {
-        "Alexander Larsson",
-        "Ali Abdin",
-        "Anders Carlsson",
-        "Andrew Walton",
-        "Andy Hertzfeld",
-        "Arlo Rose",
-        "Christian Neumair",
-        "Cosimo Cecchi",
-        "Darin Adler",
-        "David Camp",
-        "Eli Goldberg",
-        "Elliot Lee",
-        "Eskil Heyn Olsen",
-        "Ettore Perazzoli",
-        "Gene Z. Ragan",
-        "George Lebl",
-        "Ian McKellar",
-        "J Shane Culpepper",
-        "James Willcox",
-        "Jan Arne Petersen",
-        "John Harper",
-        "John Sullivan",
-        "Josh Barrow",
-        "Maciej Stachowiak",
-        "Mark McLoughlin",
-        "Mathieu Lacage",
-        "Mike Engber",
-        "Mike Fleming",
-        "Pavel Cisler",
-        "Ramiro Estrugo",
-        "Raph Levien",
-        "Rebecca Schulman",
-        "Robey Pointer",
-        "Robin * Slomkowski",
-        "Seth Nickell",
-        "Susan Kare",
-        "Tomas Bzatek",
-        "William Jon McCann",
+        "The contributors to the Nautilus project",
         NULL
     };
     const gchar *documenters[] =
@@ -3001,7 +2848,7 @@ nautilus_window_show_about_dialog (NautilusWindow *window)
                            "version", VERSION,
                            "comments", _("Access and organize your files"),
                            "website", "https://wiki.gnome.org/action/show/Apps/Files",
-                           "copyright", "© 1999–2018 The Files Authors",
+                           "copyright", "© 1999 The Files Authors",
                            "license-type", GTK_LICENSE_GPL_3_0,
                            "artists", artists,
                            "authors", authors,
@@ -3031,70 +2878,3 @@ nautilus_window_search (NautilusWindow *window,
         g_warning ("Trying search on a slot but no active slot present");
     }
 }
-
-/* Ideally, this method should be a simple wrapper for the slot method. However,
- * going back or forward can result in a new slot (or another subclass), so we
- * workaround that by duplicating part of nautilus_window_slot_back_or_forward()
- */
-void
-nautilus_window_back_or_forward (NautilusWindow          *window,
-                                 gboolean                 back,
-                                 guint                    distance,
-                                 NautilusWindowOpenFlags  flags)
-{
-    NautilusWindowSlot *slot;
-    GList *next_location_list, *back_list, *forward_list;
-    GFile *next_location;
-    guint len;
-    NautilusBookmark *next_location_bookmark;
-    gboolean active_slot_handles_location;
-
-    slot = nautilus_window_get_active_slot (window);
-    back_list = nautilus_window_slot_get_back_history (slot);
-    forward_list = nautilus_window_slot_get_forward_history (slot);
-
-    next_location_list = back ? back_list : forward_list;
-
-    len = (guint) g_list_length (next_location_list);
-
-    /* If we can't move in the direction at all, just return. */
-    if (len == 0)
-    {
-        return;
-    }
-
-    /* If the distance to move is off the end of the list, go to the end
-     *  of the list. */
-    if (distance >= len)
-    {
-        distance = len - 1;
-    }
-
-    next_location_bookmark = g_list_nth_data (next_location_list, distance);
-    next_location = nautilus_bookmark_get_location (next_location_bookmark);
-
-    active_slot_handles_location = nautilus_window_slot_handles_location (slot, next_location);
-
-    if (!active_slot_handles_location)
-    {
-        NautilusNavigationState *navigation_state;
-        NautilusLocationChangeType location_change_type;
-
-        navigation_state = nautilus_window_slot_get_navigation_state (slot);
-
-        location_change_type = back ? NAUTILUS_LOCATION_CHANGE_BACK : NAUTILUS_LOCATION_CHANGE_FORWARD;
-
-        slot = replace_active_slot (window, next_location, flags);
-
-        nautilus_window_slot_open_location_set_navigation_state (slot,
-                                                                 next_location, flags, NULL,
-                                                                 location_change_type,
-                                                                 navigation_state, distance);
-
-        free_navigation_state (navigation_state);
-    }
-    else
-    {
-        nautilus_window_slot_back_or_forward (slot, back, distance, flags);
-    }
-}
